
from odoo.exceptions import UserError
from odoo.tests import new_test_user

from odoo.tests.common import TransactionCase, tagged


@tagged("post_install", "-at_install")
class TestPageGroup(TransactionCase):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        Page = cls.env['website.page']

        cls.user_editor = new_test_user(
            env=cls.env,
            login='editor',
        )
        cls.user_editor.write({
            'groups_id': [
                (6, 0, cls.env.ref('base.group_user').ids),
                (3, cls.env.ref('website.group_website_designer').id),
                (4, cls.env.ref('website.group_website_restricted_editor').id),
            ],
        })

        cls.page_1 = Page.create({'url': '/1', 'arch': '<root>1</root>', 'type': 'qweb'})
        cls.view_1 = cls.page_1.view_id
        cls.page_2 = Page.create({'url': '/2', 'arch': '<root>2</root>', 'type': 'qweb'})
        cls.view_2 = cls.page_2.view_id
        cls.view_3 = Page.view_id.create({'name': '/3', 'arch': '<root>3</root>', 'type': 'qweb'})

        cls.page_group_1 = cls.env['website.page.group'].create({
            'name': '1',
            'description': 'can only update page_1',
            'user_ids': [(6, 0, cls.user_editor.ids)],
            'page_ids': [(6, 0, cls.page_1.ids)],
        })

        cls.page_group_2 = cls.env['website.page.group'].create({
            'name': '2',
            'description': 'can only update page_2',
            'user_ids': [(6, 0, [1])],  # not user_editor
            'page_ids': [(6, 0, cls.page_2.ids)],
        })

    def test_01_can_save_view(self):
        with self.assertRaises(UserError):
            # cannot write if not using method save
            self.view_1.with_user(user=self.user_editor).write({
                'arch': '<data></data>'
            })

        # page_ids is not readable due to ir.rule btw
        assert len(self.view_1.page_ids)
        self.view_1.with_user(user=self.user_editor).save('<data>page1</data>', xpath='.')

        with self.assertRaises(UserError):
            # page_ids is not readable due to ir.rule btw
            assert not len(self.view_2.with_user(user=self.user_editor).page_ids)
            self.view_2.invalidate_model(['page_ids'])
            # cache it as sudo to check that we handle the case where you have page_ids but without
            # rights or read them, so without rights to edit them...
            assert len(self.view_2.page_ids)
            assert len(self.view_2.with_user(user=self.user_editor).page_ids)

            # cannot write on view with page_ids but on wich you don't have access
            self.view_2.with_user(user=self.user_editor).save('<data>page2</data>', xpath='.')

        with self.assertRaises(UserError):
            # cannot write on view without page_ids
            assert not len(self.view_3.page_ids)
            self.view_3.with_user(user=self.user_editor).save('<data>page3</data>', xpath='.')

    def test_02_can_edit_page(self):
        # can edit page
        self.page_1.with_user(user=self.user_editor).write({
            'url': '/ok'
        })

        # cannot edit page
        with self.assertRaises(UserError):
            self.page_2.with_user(user=self.user_editor).write({
                'url': '/ok'
            })

    def test_03_can_optimize_seo(self):
        # can write seo on view_id via inherits if website_meta_* only
        self.page_1.with_user(user=self.user_editor).write({
            'website_meta_title': 'foo',
            'website_meta_description': 'bar',
        })

        with self.assertRaises(UserError):
            # can write seo on page on which you don't have access
            self.page_2.with_user(user=self.user_editor).write({
                'website_meta_title': 'foo',
                'website_meta_description': 'bar',
            })

        with self.assertRaises(UserError):
            # can write more than seo on view field on which you have access
            self.page_1.with_user(user=self.user_editor).write({
                'website_meta_title': 'foo',
                'key': '2',
            })

        with self.assertRaises(UserError):
            # can write more than seo on view field on which you don't have access
            self.page_2.with_user(user=self.user_editor).write({
                'website_meta_title': 'foo',
                'track': True,
            })
